"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var readline = require("readline");
var browser_1 = require("./browser");
var CSSInject = /** @class */ (function () {
    function CSSInject(fpath, options) {
        this.searchEnd = false;
        this.fpath = '';
        this.targetCSS = [];
        this.targetCSSMap = new Map();
        this.fpath = fpath;
        this.options = options;
        this.parse();
    }
    CSSInject.clean = function () {
        if (CSSInject.queue.length) {
            CSSInject.queue.forEach(function (i) { return i.cancel(); });
            CSSInject.queue.length = 0;
        }
    };
    CSSInject.init = function (items, options) {
        CSSInject.clean();
        CSSInject.queue = items.map(function (fPath) { return new CSSInject(fPath, options); });
        return Array.from(CSSInject.queue);
    };
    CSSInject.use = function (p) {
        CSSInject.parser = p;
    };
    CSSInject.prototype.parse = function () {
        var _this = this;
        var inputStream = this.stream = fs.createReadStream(this.fpath);
        var rl = this.rl = readline.createInterface({
            input: inputStream,
            crlfDelay: Infinity,
        });
        rl.on('line', function (line) {
            // only top directive will parse
            if (!line.startsWith('///')) {
                _this.close();
                return;
            }
            var result = CSSInject.directiveRe.exec(line);
            if (result) {
                _this.targetCSS.push(result[1]);
            }
        });
    };
    CSSInject.prototype.close = function () {
        var _this = this;
        this.stream.close();
        this.rl.close();
        if (!this.targetCSS.length) {
            this.searchEnd = true;
        }
        else {
            Promise.all(this.targetCSS.map(function (p) { return CSSInject.parser(p, _this.options); })).then(function (css) {
                _this.targetCSS.forEach(function (p, index) {
                    _this.targetCSSMap.set(p, css[index]);
                });
                _this.searchEnd = true;
            });
        }
    };
    CSSInject.prototype.cancel = function () {
        this.targetCSS.length = 0;
        this.close();
    };
    CSSInject.prototype.write = function () {
        var _this = this;
        var e_1, _a;
        if (this.searchEnd && !this.targetCSS.length) {
            return Promise.resolve();
        }
        if (!this.searchEnd) {
            return new Promise(function (resolve) {
                setImmediate(function () { resolve(_this.write()); });
            });
        }
        try {
            for (var _b = __values(this.targetCSSMap), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), target = _d[0], css = _d[1];
                var distPath = this.fpath
                    .replace(this.options.rootDir, this.options.outDir)
                    .replace(/\.tsx?$/, '.js');
                fs.writeFileSync(distPath, browser_1.inject(css), {
                    flag: 'a',
                });
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        // tslint:disable-next-line
        console.log('css injected');
        return Promise.resolve();
    };
    CSSInject.queue = [];
    CSSInject.directiveRe = /<css-inject path="(.+?)"/;
    return CSSInject;
}());
exports.CSSInject = CSSInject;
