"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var child_process_1 = require("child_process");
var CSSInject_1 = require("./CSSInject");
var less_1 = require("./css-processors/less");
var utils_1 = require("./utils");
// tslint:disable-next-line
var debounce = require("lodash.debounce");
var base = process.cwd();
var configPath = path.resolve(process.cwd(), 'tsconfig.json');
if (!configPath) {
    console.error('can\'t find tsconfig.json');
    process.exit(1);
}
var compilerOptions = JSON.parse(fs.readFileSync(configPath).toString()).compilerOptions;
CSSInject_1.CSSInject.use(less_1.parse);
var config = {
    rootDir: path.resolve(base, compilerOptions.rootDir || 'src'),
    outDir: path.resolve(base, compilerOptions.outDir || 'dist'),
};
function findTarget(baseDir) {
    var result = [];
    var files = fs.readdirSync(baseDir, { withFileTypes: true });
    files.forEach(function (f) {
        if (f.isFile() && /\.tsx?$/.test(f.name)) {
            result.push(path.resolve(baseDir, f.name));
        }
        else if (f.isDirectory()) {
            result = result.concat(findTarget(path.resolve(baseDir, f.name)));
        }
    });
    return result;
}
function compile(watchMode) {
    if (watchMode === void 0) { watchMode = false; }
    var targets = findTarget(config.rootDir);
    var injections = CSSInject_1.CSSInject.init(targets, config);
    var tsc = utils_1.getTsc();
    if (!tsc) {
        console.error('can\'t found tsc');
        process.exit(1);
    }
    child_process_1.exec(tsc + " -p .", {
        cwd: base,
    }, function (err, stdout) {
        if (err) {
            console.error(stdout);
            if (watchMode) {
                // tslint:disable-next-line
                console.log('waiting for change');
                return;
            }
            else {
                console.error(err);
                process.exit(1);
            }
        }
        Promise.all(injections.map(function (item) { return item.write(); }))
            .then(function () {
            // tslint:disable-next-line
            console.log('compiled\n');
        });
    });
}
exports.compile = compile;
function watchCompile() {
    compile(true);
    fs.watch(config.rootDir, {
        recursive: true,
    }, debounce(function () {
        // tslint:disable-next-line
        console.log('recompile');
        compile(true);
    }, 100));
}
exports.watchCompile = watchCompile;
